# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dti import FSLXCommand


def test_FSLXCommand_inputs():
    input_map = dict(
        all_ard=dict(
            argstr="--allard",
            xor=("no_ard", "all_ard"),
        ),
        args=dict(
            argstr="%s",
        ),
        burn_in=dict(
            argstr="--burnin=%d",
            usedefault=True,
        ),
        burn_in_no_ard=dict(
            argstr="--burnin_noard=%d",
            usedefault=True,
        ),
        bvals=dict(
            argstr="--bvals=%s",
            extensions=None,
            mandatory=True,
        ),
        bvecs=dict(
            argstr="--bvecs=%s",
            extensions=None,
            mandatory=True,
        ),
        cnlinear=dict(
            argstr="--cnonlinear",
            xor=("no_spat", "non_linear", "cnlinear"),
        ),
        dwi=dict(
            argstr="--data=%s",
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        f0_ard=dict(
            argstr="--f0 --ardf0",
            xor=["f0_noard", "f0_ard", "all_ard"],
        ),
        f0_noard=dict(
            argstr="--f0",
            xor=["f0_noard", "f0_ard"],
        ),
        force_dir=dict(
            argstr="--forcedir",
            usedefault=True,
        ),
        fudge=dict(
            argstr="--fudge=%d",
        ),
        logdir=dict(
            argstr="--logdir=%s",
            usedefault=True,
        ),
        mask=dict(
            argstr="--mask=%s",
            extensions=None,
            mandatory=True,
        ),
        model=dict(
            argstr="--model=%d",
        ),
        n_fibres=dict(
            argstr="--nfibres=%d",
            mandatory=True,
            usedefault=True,
        ),
        n_jumps=dict(
            argstr="--njumps=%d",
            usedefault=True,
        ),
        no_ard=dict(
            argstr="--noard",
            xor=("no_ard", "all_ard"),
        ),
        no_spat=dict(
            argstr="--nospat",
            xor=("no_spat", "non_linear", "cnlinear"),
        ),
        non_linear=dict(
            argstr="--nonlinear",
            xor=("no_spat", "non_linear", "cnlinear"),
        ),
        output_type=dict(),
        rician=dict(
            argstr="--rician",
        ),
        sample_every=dict(
            argstr="--sampleevery=%d",
            usedefault=True,
        ),
        seed=dict(
            argstr="--seed=%d",
        ),
        update_proposal_every=dict(
            argstr="--updateproposalevery=%d",
            usedefault=True,
        ),
    )
    inputs = FSLXCommand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FSLXCommand_outputs():
    output_map = dict(
        dyads=dict(),
        fsamples=dict(),
        mean_S0samples=dict(
            extensions=None,
        ),
        mean_dsamples=dict(
            extensions=None,
        ),
        mean_fsamples=dict(),
        mean_tausamples=dict(
            extensions=None,
        ),
        phsamples=dict(),
        thsamples=dict(),
    )
    outputs = FSLXCommand.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""
Using nipype with persistence and lazy recomputation but without explicit
name-steps pipeline: getting back scope in command-line based programming.
"""
import os
import hashlib
import pickle
import time
import shutil
import glob

from ..interfaces.base import BaseInterface
from ..pipeline.engine import Node
from ..pipeline.engine.utils import modify_paths

###############################################################################
# PipeFunc object: callable interface to nipype.interface objects


class PipeFunc(object):
    """Callable interface to nipype.interface objects

    Use this to wrap nipype.interface object and call them
    specifying their input with keyword arguments::

        fsl_merge = PipeFunc(fsl.Merge, base_dir='.')
        out = fsl_merge(in_files=files, dimension='t')
    """

    def __init__(self, interface, base_dir, callback=None):
        """

        Parameters
        ===========
        interface: a nipype interface class
            The interface class to wrap
        base_dir: a string
            The directory in which the computation will be
            stored
        callback: a callable
            An optional callable called each time after the function
            is called.
        """
        if not (isinstance(interface, type) and issubclass(interface, BaseInterface)):
            raise ValueError(
                "the interface argument should be a nipype "
                "interface class, but %s (type %s) was passed."
                % (interface, type(interface))
            )
        self.interface = interface
        base_dir = os.path.abspath(base_dir)
        if not os.path.exists(base_dir) and os.path.isdir(base_dir):
            raise ValueError("base_dir should be an existing directory")
        self.base_dir = base_dir
        doc = "%s\n%s" % (self.interface.__doc__, self.interface.help(returnhelp=True))
        self.__doc__ = doc
        self.callback = callback

    def __call__(self, **kwargs):
        kwargs = modify_paths(kwargs, relative=False)
        interface = self.interface()
        # Set the inputs early to get some argument checking
        interface.inputs.trait_set(**kwargs)
        # Make a name for our node
        inputs = interface.inputs.get_hashval()
        hasher = hashlib.new("md5")
        hasher.update(pickle.dumps(inputs))
        dir_name = "%s-%s" % (
            interface.__class__.__module__.replace(".", "-"),
            interface.__class__.__name__,
        )
        job_name = hasher.hexdigest()
        node = Node(interface, name=job_name)
        node.base_dir = os.path.join(self.base_dir, dir_name)

        cwd = os.getcwd()
        try:
            out = node.run()
        finally:
            # node.run() changes to the node directory - if something goes
            # wrong before it cds back you would end up in strange places
            os.chdir(cwd)
        if self.callback is not None:
            self.callback(dir_name, job_name)
        return out

    def __repr__(self):
        return "{}({}.{}), base_dir={})".format(
            self.__class__.__name__,
            self.interface.__module__,
            self.interface.__name__,
            self.base_dir,
        )


###############################################################################
# Memory manager: provide some tracking about what is computed when, to
# be able to flush the disk


def read_log(filename, run_dict=None):
    if run_dict is None:
        run_dict = dict()

    with open(filename, "r") as logfile:
        for line in logfile:
            dir_name, job_name = line[:-1].split("/")
            jobs = run_dict.get(dir_name, set())
            jobs.add(job_name)
            run_dict[dir_name] = jobs
    return run_dict


def rm_all_but(base_dir, dirs_to_keep, warn=False):
    """Remove all the sub-directories of base_dir, but those listed

    Parameters
    ============
    base_dir: string
        The base directory
    dirs_to_keep: set
        The names of the directories to keep
    """
    try:
        all_dirs = os.listdir(base_dir)
    except OSError:
        "Dir has been deleted"
        return
    all_dirs = [d for d in all_dirs if not d.startswith("log.")]
    dirs_to_rm = list(dirs_to_keep.symmetric_difference(all_dirs))
    for dir_name in dirs_to_rm:
        dir_name = os.path.join(base_dir, dir_name)
        if os.path.exists(dir_name):
            if warn:
                print("removing directory: %s" % dir_name)
            shutil.rmtree(dir_name)


class _MemoryCallback(object):
    "An object to avoid closures and have everything pickle"

    def __init__(self, memory):
        self.memory = memory

    def __call__(self, dir_name, job_name):
        self.memory._log_name(dir_name, job_name)


class Memory(object):
    """Memory context to provide caching for interfaces

    Parameters
    ==========
    base_dir: string
        The directory name of the location for the caching

    Methods
    =======
    cache
        Creates a cacheable function from an nipype Interface class
    clear_previous_runs
        Removes from the disk all the runs that where not used after
        the creation time of the specific Memory instance
    clear_previous_runs
        Removes from the disk all the runs that where not used after
        the given time
    """

    def __init__(self, base_dir):
        base_dir = os.path.join(os.path.abspath(base_dir), "nipype_mem")
        if not os.path.exists(base_dir):
            os.mkdir(base_dir)
        elif not os.path.isdir(base_dir):
            raise ValueError("base_dir should be a directory")
        self.base_dir = base_dir
        open(os.path.join(base_dir, "log.current"), "a").close()

    def cache(self, interface):
        """Returns a callable that caches the output of an interface

        Parameters
        ==========
        interface: nipype interface
            The nipype interface class to be wrapped and cached

        Returns
        =======
        pipe_func: a PipeFunc callable object
            An object that can be used as a function to apply the
            interface to arguments. Inputs of the interface are given
            as keyword arguments, bearing the same name as the name
            in the inputs specs of the interface.

        Examples
        ========

        >>> from tempfile import mkdtemp
        >>> mem = Memory(mkdtemp())
        >>> from nipype.interfaces import fsl

        Here we create a callable that can be used to apply an
        fsl.Merge interface to files

        >>> fsl_merge = mem.cache(fsl.Merge)

        Now we apply it to a list of files. We need to specify the
        list of input files and the dimension along which the files
        should be merged.

        >>> results = fsl_merge(in_files=['a.nii', 'b.nii'],
        ...                     dimension='t') # doctest: +SKIP

        We can retrieve the resulting file from the outputs:
        >>> results.outputs.merged_file # doctest: +SKIP
        '...'
        """
        return PipeFunc(interface, self.base_dir, _MemoryCallback(self))

    def _log_name(self, dir_name, job_name):
        """Increment counters tracking which cached function get executed."""
        base_dir = self.base_dir
        # Every counter is a file opened in append mode and closed
        # immediately to avoid race conditions in parallel computing:
        # file appends are atomic
        with open(os.path.join(base_dir, "log.current"), "a") as currentlog:
            currentlog.write("%s/%s\n" % (dir_name, job_name))

        t = time.localtime()
        year_dir = os.path.join(base_dir, "log.%i" % t.tm_year)
        try:
            os.mkdir(year_dir)
        except OSError:
            "Dir exists"
        month_dir = os.path.join(year_dir, "%02i" % t.tm_mon)
        try:
            os.mkdir(month_dir)
        except OSError:
            "Dir exists"

        with open(os.path.join(month_dir, "%02i.log" % t.tm_mday), "a") as rotatefile:
            rotatefile.write("%s/%s\n" % (dir_name, job_name))

    def clear_previous_runs(self, warn=True):
        """Remove all the cache that where not used in the latest run of
        the memory object: i.e. since the corresponding Python object
        was created.

        Parameters
        ==========
        warn: boolean, optional
            If true, echoes warning messages for all directory
            removed
        """
        base_dir = self.base_dir
        latest_runs = read_log(os.path.join(base_dir, "log.current"))
        self._clear_all_but(latest_runs, warn=warn)

    def clear_runs_since(self, day=None, month=None, year=None, warn=True):
        """Remove all the cache that where not used since the given date

        Parameters
        ==========
        day, month, year: integers, optional
            The integers specifying the latest day (in localtime) that
            a node should have been accessed to be kept. If not
            given, the current date is used.
        warn: boolean, optional
            If true, echoes warning messages for all directory
            removed
        """
        t = time.localtime()
        day = day if day is not None else t.tm_mday
        month = month if month is not None else t.tm_mon
        year = year if year is not None else t.tm_year
        base_dir = self.base_dir
        cut_off_file = "%s/log.%i/%02i/%02i.log" % (base_dir, year, month, day)
        logs_to_flush = list()
        recent_runs = dict()
        for log_name in glob.glob("%s/log.*/*/*.log" % base_dir):
            if log_name < cut_off_file:
                logs_to_flush.append(log_name)
            else:
                recent_runs = read_log(log_name, recent_runs)
        self._clear_all_but(recent_runs, warn=warn)
        for log_name in logs_to_flush:
            os.remove(log_name)

    def _clear_all_but(self, runs, warn=True):
        """Remove all the runs appart from those given to the function
        input.
        """
        rm_all_but(self.base_dir, set(runs.keys()), warn=warn)
        for dir_name, job_names in list(runs.items()):
            rm_all_but(os.path.join(self.base_dir, dir_name), job_names, warn=warn)

    def __repr__(self):
        return "{}(base_dir={})".format(self.__class__.__name__, self.base_dir)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Utilities to pull in documentation from command-line tools.

Examples
--------

# Instantiate bet object
from nipype.interfaces import fsl
from nipype.utils import docparse
better = fsl.Bet()
docstring = docparse.get_doc(better.cmd, better.opt_map)

"""
import subprocess
from ..interfaces.base import CommandLine
from .misc import is_container


def grab_doc(cmd, trap_error=True):
    """Run cmd without args and grab documentation.

    Parameters
    ----------
    cmd : string
        Command line string
    trap_error : boolean
        Ensure that returncode is 0

    Returns
    -------
    doc : string
        The command line documentation
    """

    proc = subprocess.Popen(
        cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
    )
    stdout, stderr = proc.communicate()

    if trap_error and proc.returncode:
        msg = "Attempting to run %s. Returned Error: %s" % (cmd, stderr)
        raise IOError(msg)

    if stderr:
        # A few programs, like fast and fnirt, send their help to
        # stderr instead of stdout.
        # XXX: Test for error vs. doc in stderr
        return stderr
    return stdout


def reverse_opt_map(opt_map):
    """Reverse the key/value pairs of the option map in the interface classes.

    Parameters
    ----------
    opt_map : dict
        Dictionary mapping the attribute name to a command line flag.
        Each interface class defines these for the command it wraps.

    Returns
    -------
    rev_opt_map : dict
       Dictionary mapping the flags to the attribute name.
    """

    # For docs, we only care about the mapping from our attribute
    # names to the command-line flags.  The 'v.split()[0]' below
    # strips off the string format characters.
    # if (k != 'flags' and v) , key must not be flags as it is generic,
    # v must not be None or it cannot be parsed by this line
    revdict = {}
    for key, value in list(opt_map.items()):
        if is_container(value):
            # The value is a tuple where the first element is the
            # format string and the second element is a docstring.
            value = value[0]
        if key != "flags" and value is not None:
            revdict[value.split()[0]] = key
    return revdict


def format_params(paramlist, otherlist=None):
    """Format the parameters according to the nipy style conventions.

    Since the external programs do not conform to any conventions, the
    resulting docstrings are not ideal.  But at a minimum the
    Parameters section is reasonably close.

    Parameters
    ----------
    paramlist : list
        List of strings where each list item matches exactly one
        parameter and it's description.  These items will go into the
        'Parameters' section of the docstring.
    otherlist : list
        List of strings, similar to paramlist above.  These items will
        go into the 'Other Parameters' section of the docstring.

    Returns
    -------
    doc : string
        The formatted docstring.
    """

    hdr = "Parameters"
    delim = "----------"
    paramlist.insert(0, delim)
    paramlist.insert(0, hdr)
    params = "\n".join(paramlist)
    otherparams = []
    doc = "".join(params)
    if otherlist:
        hdr = "Others Parameters"
        delim = "-----------------"
        otherlist.insert(0, delim)
        otherlist.insert(0, hdr)
        otherlist.insert(0, "\n")
        otherparams = "\n".join(otherlist)
        doc = "".join([doc, otherparams])
    return doc


def insert_doc(doc, new_items):
    """Insert ``new_items`` into the beginning of the ``doc``

    Docstrings in ``new_items`` will be inserted right after the
    *Parameters* header but before the existing docs.

    Parameters
    ----------
    doc : str
        The existing docstring we're inserting docmentation into.
    new_items : list
        List of strings to be inserted in the ``doc``.

    Examples
    --------
    >>> from nipype.utils.docparse import insert_doc
    >>> doc = '''Parameters
    ... ----------
    ... outline :
    ...     something about an outline'''

    >>> new_items = ['infile : str', '    The name of the input file']
    >>> new_items.extend(['outfile : str', '    The name of the output file'])
    >>> newdoc = insert_doc(doc, new_items)
    >>> print(newdoc)
    Parameters
    ----------
    infile : str
        The name of the input file
    outfile : str
        The name of the output file
    outline :
        something about an outline

    """

    # Insert new_items after the Parameters header
    doclist = doc.split("\n")
    tmpdoc = doclist[:2]
    # Add new_items
    tmpdoc.extend(new_items)
    # Add rest of documents
    tmpdoc.extend(doclist[2:])
    # Insert newlines
    newdoc = []
    for line in tmpdoc:
        newdoc.append(line)
        newdoc.append("\n")
    # We add one too many newlines, remove it.
    newdoc.pop(-1)
    return "".join(newdoc)


def build_doc(doc, opts):
    """Build docstring from doc and options

    Parameters
    ----------
    rep_doc : string
        Documentation string
    opts : dict
        Dictionary of option attributes and keys.  Use reverse_opt_map
        to reverse flags and attrs from opt_map class attribute.

    Returns
    -------
    newdoc : string
        The docstring with flags replaced with attribute names and
        formated to match nipy standards (as best we can).

    """

    # Split doc into line elements.  Generally, each line is an
    # individual flag/option.
    doclist = doc.split("\n")
    newdoc = []
    flags_doc = []
    for line in doclist:
        linelist = line.split()
        if not linelist:
            # Probably an empty line
            continue
        # For lines we care about, the first item is the flag
        if "," in linelist[0]:  # sometimes flags are only seperated by comma
            flag = linelist[0].split(",")[0]
        else:
            flag = linelist[0]
        attr = opts.get(flag)
        if attr is not None:
            # newline = line.replace(flag, attr)
            # Replace the flag with our attribute name
            linelist[0] = "%s :\n    " % str(attr)
            # Add some line formatting
            newline = " ".join(linelist)
            newdoc.append(newline)
        else:
            if line[0].isspace():
                # For all the docs I've looked at, the flags all have
                # indentation (spaces) at the start of the line.
                # Other parts of the docs, like 'usage' statements
                # start with alpha-numeric characters.  We only care
                # about the flags.
                flags_doc.append(line)
    return format_params(newdoc, flags_doc)


def get_doc(cmd, opt_map, help_flag=None, trap_error=True):
    """Get the docstring from our command and options map.

    Parameters
    ----------
    cmd : string
        The command whose documentation we are fetching
    opt_map : dict
        Dictionary of flags and option attributes.
    help_flag : string
        Provide additional help flag. e.g., -h
    trap_error : boolean
        Override if underlying command returns a non-zero returncode

    Returns
    -------
    doc : string
        The formated docstring

    """
    res = CommandLine(
        "which %s" % cmd.split(" ")[0],
        resource_monitor=False,
        terminal_output="allatonce",
    ).run()
    cmd_path = res.runtime.stdout.strip()
    if cmd_path == "":
        raise Exception("Command %s not found" % cmd.split(" ")[0])
    if help_flag:
        cmd = " ".join((cmd, help_flag))
    doc = grab_doc(cmd, trap_error)
    opts = reverse_opt_map(opt_map)
    return build_doc(doc, opts)


def _parse_doc(doc, style=["--"]):
    """Parses a help doc for inputs

    Parameters
    ----------
    doc : string
        Documentation string
    style : string default ['--']
        The help command style (--, -)

    Returns
    -------
    optmap : dict of input parameters
    """

    # Split doc into line elements.  Generally, each line is an
    # individual flag/option.
    doclist = doc.split("\n")
    optmap = {}
    if isinstance(style, (str, bytes)):
        style = [style]
    for line in doclist:
        linelist = line.split()
        flag = [
            item
            for i, item in enumerate(linelist)
            if i < 2 and any([item.startswith(s) for s in style]) and len(item) > 1
        ]
        if flag:
            if len(flag) == 1:
                style_idx = [flag[0].startswith(s) for s in style].index(True)
                flag = flag[0]
            else:
                style_idx = []
                for f in flag:
                    for i, s in enumerate(style):
                        if f.startswith(s):
                            style_idx.append(i)
                            break
                flag = flag[style_idx.index(min(style_idx))]
                style_idx = min(style_idx)
            optmap[flag.split(style[style_idx])[1]] = "%s %%s" % flag
    return optmap


def get_params_from_doc(cmd, style="--", help_flag=None, trap_error=True):
    """Auto-generate option map from command line help

    Parameters
    ----------
    cmd : string
        The command whose documentation we are fetching
    style : string default ['--']
        The help command style (--, -). Multiple styles can be provided in a
        list e.g. ['--','-'].
    help_flag : string
        Provide additional help flag. e.g., -h
    trap_error : boolean
        Override if underlying command returns a non-zero returncode

    Returns
    -------
    optmap : dict
        Contains a mapping from input to command line variables

    """
    res = CommandLine(
        "which %s" % cmd.split(" ")[0],
        resource_monitor=False,
        terminal_output="allatonce",
    ).run()
    cmd_path = res.runtime.stdout.strip()
    if cmd_path == "":
        raise Exception("Command %s not found" % cmd.split(" ")[0])
    if help_flag:
        cmd = " ".join((cmd, help_flag))
    doc = grab_doc(cmd, trap_error)
    return _parse_doc(doc, style)


def replace_opts(rep_doc, opts):
    """Replace flags with parameter names.

    This is a simple operation where we replace the command line flags
    with the attribute names.

    Parameters
    ----------
    rep_doc : string
        Documentation string
    opts : dict
        Dictionary of option attributes and keys.  Use reverse_opt_map
        to reverse flags and attrs from opt_map class attribute.

    Returns
    -------
    rep_doc : string
        New docstring with flags replaces with attribute names.

    Examples
    --------
    doc = grab_doc('bet')
    opts = reverse_opt_map(fsl.Bet.opt_map)
    rep_doc = replace_opts(doc, opts)

    """

    # Replace flags with attribute names
    for key, val in list(opts.items()):
        rep_doc = rep_doc.replace(key, val)
    return rep_doc



"""Test afni deconvolve"""

from ..model import Deconvolve


def test_x1dstop():
    deconv = Deconvolve()
    deconv.inputs.out_file = "file.nii"
    assert "out_file" in deconv._list_outputs()
    deconv.inputs.x1D_stop = True
    assert "out_file" not in deconv._list_outputs()
    assert "cbucket" not in deconv._list_outputs()



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from ..base import CommandLineInputSpec, CommandLine, TraitedSpec, File


class Vnifti2ImageInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True, argstr="-in %s", mandatory=True, position=1, desc="in file"
    )
    attributes = File(exists=True, argstr="-attr %s", position=2, desc="attribute file")
    out_file = File(
        name_template="%s.v",
        keep_extension=False,
        argstr="-out %s",
        hash_files=False,
        position=-1,
        desc="output data file",
        name_source=["in_file"],
    )


class Vnifti2ImageOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Output vista file")


class Vnifti2Image(CommandLine):
    """
    Convert a nifti file into a vista file.

    Example
    -------
    >>> vimage = Vnifti2Image()
    >>> vimage.inputs.in_file = 'image.nii'
    >>> vimage.cmdline
    'vnifti2image -in image.nii -out image.v'
    >>> vimage.run()  # doctest: +SKIP

    """

    _cmd = "vnifti2image"
    input_spec = Vnifti2ImageInputSpec
    output_spec = Vnifti2ImageOutputSpec


class VtoMatInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True, argstr="-in %s", mandatory=True, position=1, desc="in file"
    )
    out_file = File(
        name_template="%s.mat",
        keep_extension=False,
        argstr="-out %s",
        hash_files=False,
        position=-1,
        desc="output mat file",
        name_source=["in_file"],
    )


class VtoMatOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Output mat file")


class VtoMat(CommandLine):
    """
    Convert a nifti file into a vista file.

    Example
    -------
    >>> vimage = VtoMat()
    >>> vimage.inputs.in_file = 'image.v'
    >>> vimage.cmdline
    'vtomat -in image.v -out image.mat'
    >>> vimage.run()  # doctest: +SKIP

    """

    _cmd = "vtomat"
    input_spec = VtoMatInputSpec
    output_spec = VtoMatOutputSpec



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from ..nipype2boutiques import generate_boutiques_descriptor
from nipype.testing import example_data
import json


def test_generate():
    ignored_inputs = ["args", "environ", "output_type"]
    desc = generate_boutiques_descriptor(
        module="nipype.interfaces.fsl",
        interface_name="FLIRT",
        container_image=("mcin/" "docker-fsl:latest"),
        container_index="index.docker.io",
        container_type="docker",
        verbose=False,
        save=False,
        ignore_inputs=ignored_inputs,
        author=("Oxford Centre for Functional" " MRI of the Brain (FMRIB)"),
    )

    with open(example_data("nipype2boutiques_example.json"), "r") as desc_file:
        # Make sure that output descriptor matches the expected descriptor.
        output_desc = json.loads(desc)
        expected_desc = json.load(desc_file)
        assert output_desc.get("name") == expected_desc.get("name")
        assert output_desc.get("author") == expected_desc.get("author")
        assert output_desc.get("command-line") == expected_desc.get("command-line")
        assert output_desc.get("description") == expected_desc.get("description")
        assert len(output_desc.get("inputs")) == len(expected_desc.get("inputs"))
        assert len(output_desc.get("output-files")) == len(
            expected_desc.get("output-files")
        )
        assert output_desc.get("container-image").get("image") == expected_desc.get(
            "container-image"
        ).get("image")



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Package contains modules for generating pipelines using interfaces

"""
__docformat__ = "restructuredtext"
from .engine import Node, MapNode, JoinNode, Workflow



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for the engine module
"""
import numpy as np
import scipy.sparse as ssp


def test_scipy_sparse():
    foo = ssp.lil_matrix(np.eye(3, k=1))
    goo = foo.getrowview(0)
    goo[goo.nonzero()] = 0
    assert foo[0, 1] == 0


"""
Can use the following code to test that a mapnode crash continues successfully
Need to put this into a unit-test with a timeout

import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

wf = pe.Workflow(name='test')

def func(arg1):
    if arg1 == 2:
        raise Exception('arg cannot be ' + str(arg1))
    return arg1

funkynode = pe.MapNode(niu.Function(function=func, input_names=['arg1'],
                                    output_names=['out']),
                       iterfield=['arg1'],
                       name = 'functor')
funkynode.inputs.arg1 = [1,2]

wf.add_nodes([funkynode])
wf.base_dir = '/tmp'

wf.run(plugin='MultiProc')
"""



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..imagelabelcombine import ImageLabelCombine


def test_ImageLabelCombine_inputs():
    input_map = dict(
        InputLabelMap_A=dict(
            argstr="%s",
            extensions=None,
            position=-3,
        ),
        InputLabelMap_B=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        OutputLabelMap=dict(
            argstr="%s",
            hash_files=False,
            position=-1,
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        first_overwrites=dict(
            argstr="--first_overwrites ",
        ),
    )
    inputs = ImageLabelCombine.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ImageLabelCombine_outputs():
    output_map = dict(
        OutputLabelMap=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = ImageLabelCombine.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from nipype.interfaces.ants import registration
import os
import pytest


def test_ants_mand(tmpdir):
    tmpdir.chdir()
    filepath = os.path.dirname(os.path.realpath(__file__))
    datadir = os.path.realpath(os.path.join(filepath, "../../../testing/data"))

    ants = registration.ANTS()
    ants.inputs.transformation_model = "SyN"
    ants.inputs.moving_image = [os.path.join(datadir, "resting.nii")]
    ants.inputs.fixed_image = [os.path.join(datadir, "T1.nii")]
    ants.inputs.metric = ["MI"]

    with pytest.raises(ValueError) as er:
        ants.run()
    assert "ANTS requires a value for input 'radius'" in str(er.value)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os
from copy import deepcopy

import pytest
import pdb
from nipype.utils.filemanip import split_filename, ensure_list
from .. import preprocess as fsl
from nipype.interfaces.fsl import Info
from nipype.interfaces.base import File, TraitError, Undefined, isdefined
from nipype.interfaces.fsl import no_fsl


def fsl_name(obj, fname):
    """Create valid fsl name, including file extension for output type."""
    ext = Info.output_type_to_ext(obj.inputs.output_type)
    return fname + ext


@pytest.fixture()
def setup_infile(tmpdir):
    ext = Info.output_type_to_ext(Info.output_type())
    tmp_infile = tmpdir.join("foo" + ext)
    tmp_infile.open("w")
    return (tmp_infile.strpath, tmpdir.strpath)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_bet(setup_infile):
    tmp_infile, tp_dir = setup_infile
    # BET converts the in_file path to be relative to prevent
    # failure with long paths.
    tmp_infile = os.path.relpath(tmp_infile, start=os.getcwd())
    better = fsl.BET()
    assert better.cmd == "bet"

    # Test raising error with mandatory args absent
    with pytest.raises(ValueError):
        better.run()

    # Test generated outfile name
    better.inputs.in_file = tmp_infile
    outfile = fsl_name(better, "foo_brain")
    realcmd = "bet %s %s" % (tmp_infile, outfile)
    assert better.cmdline == realcmd
    # Test specified outfile name
    outfile = fsl_name(better, "/newdata/bar")
    better.inputs.out_file = outfile
    realcmd = "bet %s %s" % (tmp_infile, outfile)
    assert better.cmdline == realcmd

    # infile foo.nii doesn't exist
    def func():
        better.run(in_file="foo2.nii", out_file="bar.nii")

    with pytest.raises(TraitError):
        func()

    # Our options and some test values for them
    # Should parallel the opt_map structure in the class for clarity
    opt_map = {
        "outline": ("-o", True),
        "mask": ("-m", True),
        "skull": ("-s", True),
        "no_output": ("-n", True),
        "frac": ("-f 0.40", 0.4),
        "vertical_gradient": ("-g 0.75", 0.75),
        "radius": ("-r 20", 20),
        "center": ("-c 54 75 80", [54, 75, 80]),
        "threshold": ("-t", True),
        "mesh": ("-e", True),
        "surfaces": ("-A", True)
        # 'verbose': ('-v', True),
        # 'flags': ('--i-made-this-up', '--i-made-this-up'),
    }
    # Currently we don't test -R, -S, -B, -Z, -F, -A or -A2

    # test each of our arguments
    better = fsl.BET()
    outfile = fsl_name(better, "foo_brain")
    for name, settings in list(opt_map.items()):
        better = fsl.BET(**{name: settings[1]})
        # Add mandatory input
        better.inputs.in_file = tmp_infile
        realcmd = " ".join([better.cmd, tmp_infile, outfile, settings[0]])
        assert better.cmdline == realcmd


# test fast


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fast(setup_infile):
    tmp_infile, tp_dir = setup_infile
    faster = fsl.FAST()
    faster.inputs.verbose = True
    fasted = fsl.FAST(in_files=tmp_infile, verbose=True)
    fasted2 = fsl.FAST(in_files=[tmp_infile, tmp_infile], verbose=True)

    assert faster.cmd == "fast"
    assert faster.inputs.verbose
    assert faster.inputs.manual_seg == Undefined
    assert faster.inputs != fasted.inputs
    assert fasted.cmdline == "fast -v -S 1 %s" % (tmp_infile)
    assert fasted2.cmdline == "fast -v -S 2 %s %s" % (tmp_infile, tmp_infile)

    faster = fsl.FAST()
    faster.inputs.in_files = tmp_infile
    assert faster.cmdline == "fast -S 1 %s" % (tmp_infile)
    faster.inputs.in_files = [tmp_infile, tmp_infile]
    assert faster.cmdline == "fast -S 2 %s %s" % (tmp_infile, tmp_infile)

    # Our options and some test values for them
    # Should parallel the opt_map structure in the class for clarity
    opt_map = {
        "number_classes": ("-n 4", 4),
        "bias_iters": ("-I 5", 5),
        "bias_lowpass": ("-l 15", 15),
        "img_type": ("-t 2", 2),
        "init_seg_smooth": ("-f 0.035", 0.035),
        "segments": ("-g", True),
        "init_transform": ("-a %s" % (tmp_infile), "%s" % (tmp_infile)),
        "other_priors": (
            "-A %s %s %s" % (tmp_infile, tmp_infile, tmp_infile),
            (["%s" % (tmp_infile), "%s" % (tmp_infile), "%s" % (tmp_infile)]),
        ),
        "no_pve": ("--nopve", True),
        "output_biasfield": ("-b", True),
        "output_biascorrected": ("-B", True),
        "no_bias": ("-N", True),
        "out_basename": ("-o fasted", "fasted"),
        "use_priors": ("-P", True),
        "segment_iters": ("-W 14", 14),
        "mixel_smooth": ("-R 0.25", 0.25),
        "iters_afterbias": ("-O 3", 3),
        "hyper": ("-H 0.15", 0.15),
        "verbose": ("-v", True),
        "manual_seg": ("-s %s" % (tmp_infile), "%s" % (tmp_infile)),
        "probability_maps": ("-p", True),
    }

    # test each of our arguments
    for name, settings in list(opt_map.items()):
        faster = fsl.FAST(in_files=tmp_infile, **{name: settings[1]})
        assert faster.cmdline == " ".join(
            [faster.cmd, settings[0], "-S 1 %s" % tmp_infile]
        )


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fast_list_outputs(setup_infile, tmpdir):
    """By default (no -o), FSL's fast command outputs files into the same
    directory as the input files. If the flag -o is set, it outputs files into
    the cwd"""

    def _run_and_test(opts, output_base):
        outputs = fsl.FAST(**opts)._list_outputs()
        for output in outputs.values():
            if output:
                for filename in ensure_list(output):
                    assert os.path.realpath(filename).startswith(
                        os.path.realpath(output_base)
                    )

    # set up
    tmp_infile, indir = setup_infile
    cwd = tmpdir.mkdir("new")
    cwd.chdir()
    assert indir != cwd.strpath
    out_basename = "a_basename"

    # run and test
    opts = {"in_files": tmp_infile}
    input_path, input_filename, input_ext = split_filename(tmp_infile)
    _run_and_test(opts, os.path.join(input_path, input_filename))

    opts["out_basename"] = out_basename
    _run_and_test(opts, os.path.join(cwd.strpath, out_basename))


@pytest.fixture()
def setup_flirt(tmpdir):
    ext = Info.output_type_to_ext(Info.output_type())
    infile = tmpdir.join("infile" + ext)
    infile.open("w")
    reffile = tmpdir.join("reffile" + ext)
    reffile.open("w")
    return (tmpdir, infile.strpath, reffile.strpath)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_flirt(setup_flirt):
    # setup
    tmpdir, infile, reffile = setup_flirt

    flirter = fsl.FLIRT()
    assert flirter.cmd == "flirt"

    flirter.inputs.bins = 256
    flirter.inputs.cost = "mutualinfo"

    flirted = fsl.FLIRT(
        in_file=infile,
        reference=reffile,
        out_file="outfile",
        out_matrix_file="outmat.mat",
        bins=256,
        cost="mutualinfo",
    )

    flirt_est = fsl.FLIRT(
        in_file=infile,
        reference=reffile,
        out_matrix_file="outmat.mat",
        bins=256,
        cost="mutualinfo",
    )
    assert flirter.inputs != flirted.inputs
    assert flirted.inputs != flirt_est.inputs

    assert flirter.inputs.bins == flirted.inputs.bins
    assert flirter.inputs.cost == flirt_est.inputs.cost
    realcmd = (
        "flirt -in %s -ref %s -out outfile -omat outmat.mat "
        "-bins 256 -cost mutualinfo" % (infile, reffile)
    )
    assert flirted.cmdline == realcmd

    flirter = fsl.FLIRT()
    # infile not specified
    with pytest.raises(ValueError):
        flirter.cmdline
    flirter.inputs.in_file = infile
    # reference not specified
    with pytest.raises(ValueError):
        flirter.cmdline
    flirter.inputs.reference = reffile

    # Generate outfile and outmatrix
    pth, fname, ext = split_filename(infile)
    outfile = fsl_name(flirter, "%s_flirt" % fname)
    outmat = "%s_flirt.mat" % fname
    realcmd = "flirt -in %s -ref %s -out %s -omat %s" % (
        infile,
        reffile,
        outfile,
        outmat,
    )
    assert flirter.cmdline == realcmd

    # test apply_xfm option
    axfm = deepcopy(flirter)
    axfm.inputs.apply_xfm = True
    # in_matrix_file or uses_qform must be defined
    with pytest.raises(RuntimeError):
        axfm.cmdline
    axfm2 = deepcopy(axfm)
    # test uses_qform
    axfm.inputs.uses_qform = True
    assert axfm.cmdline == (realcmd + " -applyxfm -usesqform")
    # test in_matrix_file
    axfm2.inputs.in_matrix_file = reffile
    assert axfm2.cmdline == (realcmd + " -applyxfm -init %s" % reffile)

    tmpfile = tmpdir.join("file4test.nii")
    tmpfile.open("w")
    # Loop over all inputs, set a reasonable value and make sure the
    # cmdline is updated correctly.
    for key, trait_spec in sorted(fsl.FLIRT.input_spec().traits().items()):
        # Skip mandatory inputs and the trait methods
        if key in (
            "trait_added",
            "trait_modified",
            "in_file",
            "reference",
            "environ",
            "output_type",
            "out_file",
            "out_matrix_file",
            "in_matrix_file",
            "apply_xfm",
            "resource_monitor",
            "out_log",
            "save_log",
        ):
            continue
        param = None
        value = None
        if key == "args":
            param = "-v"
            value = "-v"
        elif isinstance(trait_spec.trait_type, File):
            value = tmpfile.strpath
            param = trait_spec.argstr % value
        elif trait_spec.default is False:
            param = trait_spec.argstr
            value = True
        elif key in ("searchr_x", "searchr_y", "searchr_z"):
            value = [-45, 45]
            param = trait_spec.argstr % " ".join(str(elt) for elt in value)
        else:
            value = trait_spec.default
            param = trait_spec.argstr % value
        cmdline = "flirt -in %s -ref %s" % (infile, reffile)
        # Handle autogeneration of outfile
        pth, fname, ext = split_filename(infile)
        outfile = fsl_name(fsl.FLIRT(), "%s_flirt" % fname)
        outfile = " ".join(["-out", outfile])
        # Handle autogeneration of outmatrix
        outmatrix = "%s_flirt.mat" % fname
        outmatrix = " ".join(["-omat", outmatrix])
        # Build command line
        cmdline = " ".join([cmdline, outfile, outmatrix, param])
        flirter = fsl.FLIRT(in_file=infile, reference=reffile)
        setattr(flirter.inputs, key, value)
        assert flirter.cmdline == cmdline

    # Test OutputSpec
    flirter = fsl.FLIRT(in_file=infile, reference=reffile)
    pth, fname, ext = split_filename(infile)
    flirter.inputs.out_file = "".join(["foo", ext])
    flirter.inputs.out_matrix_file = "".join(["bar", ext])
    outs = flirter._list_outputs()
    assert outs["out_file"] == os.path.join(os.getcwd(), flirter.inputs.out_file)
    assert outs["out_matrix_file"] == os.path.join(
        os.getcwd(), flirter.inputs.out_matrix_file
    )
    assert not isdefined(flirter.inputs.out_log)


# Mcflirt
@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt(setup_flirt):
    tmpdir, infile, reffile = setup_flirt

    frt = fsl.MCFLIRT()
    assert frt.cmd == "mcflirt"
    # Test generated outfile name

    frt.inputs.in_file = infile
    _, nme = os.path.split(infile)
    outfile = os.path.join(os.getcwd(), nme)
    outfile = frt._gen_fname(outfile, suffix="_mcf")
    realcmd = "mcflirt -in " + infile + " -out " + outfile
    assert frt.cmdline == realcmd
    # Test specified outfile name
    outfile2 = "/newdata/bar.nii"
    frt.inputs.out_file = outfile2
    realcmd = "mcflirt -in " + infile + " -out " + outfile2
    assert frt.cmdline == realcmd


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt_opt(setup_flirt):
    tmpdir, infile, reffile = setup_flirt
    _, nme = os.path.split(infile)

    opt_map = {
        "cost": ("-cost mutualinfo", "mutualinfo"),
        "bins": ("-bins 256", 256),
        "dof": ("-dof 6", 6),
        "ref_vol": ("-refvol 2", 2),
        "scaling": ("-scaling 6.00", 6.00),
        "smooth": ("-smooth 1.00", 1.00),
        "rotation": ("-rotation 2", 2),
        "stages": ("-stages 3", 3),
        "init": ("-init %s" % (infile), infile),
        "use_gradient": ("-gdt", True),
        "use_contour": ("-edge", True),
        "mean_vol": ("-meanvol", True),
        "stats_imgs": ("-stats", True),
        "save_mats": ("-mats", True),
        "save_plots": ("-plots", True),
    }

    for name, settings in list(opt_map.items()):
        fnt = fsl.MCFLIRT(in_file=infile, **{name: settings[1]})
        outfile = os.path.join(os.getcwd(), nme)
        outfile = fnt._gen_fname(outfile, suffix="_mcf")

        instr = "-in %s" % (infile)
        outstr = "-out %s" % (outfile)
        if name in ("init", "cost", "dof", "mean_vol", "bins"):
            assert fnt.cmdline == " ".join([fnt.cmd, instr, settings[0], outstr])
        else:
            assert fnt.cmdline == " ".join([fnt.cmd, instr, outstr, settings[0]])


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt_noinput():
    # Test error is raised when missing required args
    fnt = fsl.MCFLIRT()
    with pytest.raises(ValueError) as excinfo:
        fnt.run()
    assert str(excinfo.value).startswith("MCFLIRT requires a value for input 'in_file'")


# test fnirt


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fnirt(setup_flirt):

    tmpdir, infile, reffile = setup_flirt
    tmpdir.chdir()
    fnirt = fsl.FNIRT()
    assert fnirt.cmd == "fnirt"

    # Test list parameters
    params = [
        ("subsampling_scheme", "--subsamp", [4, 2, 2, 1], "4,2,2,1"),
        ("max_nonlin_iter", "--miter", [4, 4, 4, 2], "4,4,4,2"),
        ("ref_fwhm", "--reffwhm", [4, 2, 2, 0], "4,2,2,0"),
        ("in_fwhm", "--infwhm", [4, 2, 2, 0], "4,2,2,0"),
        ("apply_refmask", "--applyrefmask", [0, 0, 1, 1], "0,0,1,1"),
        ("apply_inmask", "--applyinmask", [0, 0, 0, 1], "0,0,0,1"),
        ("regularization_lambda", "--lambda", [0.5, 0.75], "0.5,0.75"),
        (
            "intensity_mapping_model",
            "--intmod",
            "global_non_linear",
            "global_non_linear",
        ),
    ]
    for item, flag, val, strval in params:
        fnirt = fsl.FNIRT(in_file=infile, ref_file=reffile, **{item: val})
        log = fnirt._gen_fname(infile, suffix="_log.txt", change_ext=False)
        iout = fnirt._gen_fname(infile, suffix="_warped")
        if item in ("max_nonlin_iter"):
            cmd = (
                "fnirt --in=%s "
                "--logout=%s"
                " %s=%s --ref=%s"
                " --iout=%s" % (infile, log, flag, strval, reffile, iout)
            )
        elif item in ("in_fwhm", "intensity_mapping_model"):
            cmd = "fnirt --in=%s %s=%s --logout=%s " "--ref=%s --iout=%s" % (
                infile,
                flag,
                strval,
                log,
                reffile,
                iout,
            )
        elif item.startswith("apply"):
            cmd = (
                "fnirt %s=%s "
                "--in=%s "
                "--logout=%s "
                "--ref=%s --iout=%s" % (flag, strval, infile, log, reffile, iout)
            )

        else:
            cmd = (
                "fnirt "
                "--in=%s --logout=%s "
                "--ref=%s %s=%s --iout=%s" % (infile, log, reffile, flag, strval, iout)
            )
        assert fnirt.cmdline == cmd

    # Test ValueError is raised when missing mandatory args
    fnirt = fsl.FNIRT()
    with pytest.raises(ValueError):
        fnirt.run()
    fnirt.inputs.in_file = infile
    fnirt.inputs.ref_file = reffile
    intmap_basename = "%s_intmap" % fsl.FNIRT.intensitymap_file_basename(infile)
    intmap_image = fsl_name(fnirt, intmap_basename)
    intmap_txt = "%s.txt" % intmap_basename
    # doing this to create the file to pass tests for file existence
    with open(intmap_image, "w"):
        pass
    with open(intmap_txt, "w"):
        pass

    # test files
    opt_map = [
        ("affine_file", "--aff=%s" % infile, infile),
        ("inwarp_file", "--inwarp=%s" % infile, infile),
        ("in_intensitymap_file", "--intin=%s" % intmap_basename, [intmap_image]),
        (
            "in_intensitymap_file",
            "--intin=%s" % intmap_basename,
            [intmap_image, intmap_txt],
        ),
        ("config_file", "--config=%s" % infile, infile),
        ("refmask_file", "--refmask=%s" % infile, infile),
        ("inmask_file", "--inmask=%s" % infile, infile),
        ("field_file", "--fout=%s" % infile, infile),
        ("jacobian_file", "--jout=%s" % infile, infile),
        ("modulatedref_file", "--refout=%s" % infile, infile),
        ("out_intensitymap_file", "--intout=%s" % intmap_basename, True),
        ("out_intensitymap_file", "--intout=%s" % intmap_basename, intmap_image),
        ("fieldcoeff_file", "--cout=%s" % infile, infile),
        ("log_file", "--logout=%s" % infile, infile),
    ]

    for (name, settings, arg) in opt_map:
        fnirt = fsl.FNIRT(in_file=infile, ref_file=reffile, **{name: arg})

        if name in ("config_file", "affine_file", "field_file", "fieldcoeff_file"):
            cmd = (
                "fnirt %s --in=%s "
                "--logout=%s "
                "--ref=%s --iout=%s" % (settings, infile, log, reffile, iout)
            )
        elif name in ("refmask_file"):
            cmd = (
                "fnirt --in=%s "
                "--logout=%s --ref=%s "
                "%s "
                "--iout=%s" % (infile, log, reffile, settings, iout)
            )
        elif name in (
            "in_intensitymap_file",
            "inwarp_file",
            "inmask_file",
            "jacobian_file",
        ):
            cmd = (
                "fnirt --in=%s "
                "%s "
                "--logout=%s --ref=%s "
                "--iout=%s" % (infile, settings, log, reffile, iout)
            )
        elif name in ("log_file"):
            cmd = (
                "fnirt --in=%s "
                "%s --ref=%s "
                "--iout=%s" % (infile, settings, reffile, iout)
            )
        else:
            cmd = (
                "fnirt --in=%s "
                "--logout=%s %s "
                "--ref=%s --iout=%s" % (infile, log, settings, reffile, iout)
            )

        assert fnirt.cmdline == cmd

        if name == "out_intensitymap_file":
            assert fnirt._list_outputs()["out_intensitymap_file"] == [
                intmap_image,
                intmap_txt,
            ]


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_applywarp(setup_flirt):
    tmpdir, infile, reffile = setup_flirt
    opt_map = {
        "out_file": ("--out=bar.nii", "bar.nii"),
        "premat": ("--premat=%s" % (reffile), reffile),
        "postmat": ("--postmat=%s" % (reffile), reffile),
    }

    # in_file, ref_file, field_file mandatory
    for name, settings in list(opt_map.items()):
        awarp = fsl.ApplyWarp(
            in_file=infile, ref_file=reffile, field_file=reffile, **{name: settings[1]}
        )
        if name == "out_file":
            realcmd = (
                "applywarp --in=%s "
                "--ref=%s --out=%s "
                "--warp=%s" % (infile, reffile, settings[1], reffile)
            )
        else:
            outfile = awarp._gen_fname(infile, suffix="_warp")
            realcmd = (
                "applywarp --in=%s "
                "--ref=%s --out=%s "
                "--warp=%s %s" % (infile, reffile, outfile, reffile, settings[0])
            )
        assert awarp.cmdline == realcmd


@pytest.fixture()
def setup_fugue(tmpdir):
    import nibabel as nb
    import numpy as np
    import os.path as op

    d = np.ones((80, 80, 80))
    infile = tmpdir.join("dumbfile.nii.gz").strpath
    nb.Nifti1Image(d, None, None).to_filename(infile)

    return (tmpdir, infile)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.parametrize(
    "attr, out_file",
    [
        (
            {
                "save_unmasked_fmap": True,
                "fmap_in_file": "infile",
                "mask_file": "infile",
                "output_type": "NIFTI_GZ",
            },
            "fmap_out_file",
        ),
        (
            {
                "save_unmasked_shift": True,
                "fmap_in_file": "infile",
                "dwell_time": 1.0e-3,
                "mask_file": "infile",
                "output_type": "NIFTI_GZ",
            },
            "shift_out_file",
        ),
        (
            {
                "in_file": "infile",
                "mask_file": "infile",
                "shift_in_file": "infile",
                "output_type": "NIFTI_GZ",
            },
            "unwarped_file",
        ),
    ],
)
def test_fugue(setup_fugue, attr, out_file):
    import os.path as op

    tmpdir, infile = setup_fugue

    fugue = fsl.FUGUE()
    for key, value in attr.items():
        if value == "infile":
            setattr(fugue.inputs, key, infile)
        else:
            setattr(fugue.inputs, key, value)
    res = fugue.run()

    assert isdefined(getattr(res.outputs, out_file))
    trait_spec = fugue.inputs.trait(out_file)
    out_name = trait_spec.name_template % "dumbfile"
    out_name += ".nii.gz"
    assert op.basename(getattr(res.outputs, out_file)) == out_name


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_first_genfname():
    first = fsl.FIRST()
    first.inputs.out_file = "segment.nii"
    first.inputs.output_type = "NIFTI_GZ"

    value = first._gen_fname(basename="original_segmentations")
    expected_value = os.path.abspath("segment_all_fast_origsegs.nii.gz")
    assert value == expected_value
    first.inputs.method = "none"
    value = first._gen_fname(basename="original_segmentations")
    expected_value = os.path.abspath("segment_all_none_origsegs.nii.gz")
    assert value == expected_value
    first.inputs.method = "auto"
    first.inputs.list_of_specific_structures = ["L_Hipp", "R_Hipp"]
    value = first._gen_fname(basename="original_segmentations")
    expected_value = os.path.abspath("segment_all_none_origsegs.nii.gz")
    assert value == expected_value



from ..base import BaseInterface, BaseInterfaceInputSpec, traits
from ...utils.imagemanip import copy_header as _copy_header


class CopyHeaderInputSpec(BaseInterfaceInputSpec):
    copy_header = traits.Bool(
        desc="Copy headers of the input image into the output image"
    )


class CopyHeaderInterface(BaseInterface):
    """Copy headers if the copy_header input is ``True``

    This interface mixin adds a post-run hook that allows for copying
    an input header to an output file.
    The subclass should specify a ``_copy_header_map`` that maps the **output**
    image to the **input** image whose header should be copied.

    This feature is intended for tools that are intended to adjust voxel data without
    modifying the header, but for some reason do not reliably preserve the header.

    Here we show an example interface that takes advantage of the mixin by simply
    setting the data block:

    >>> import os
    >>> import numpy as np
    >>> import nibabel as nb
    >>> from nipype.interfaces.base import SimpleInterface, TraitedSpec, File
    >>> from nipype.interfaces.mixins import CopyHeaderInputSpec, CopyHeaderInterface

    >>> class ZerofileInputSpec(CopyHeaderInputSpec):
    ...     in_file = File(mandatory=True, exists=True)

    >>> class ZerofileOutputSpec(TraitedSpec):
    ...     out_file = File()

    >>> class ZerofileInterface(SimpleInterface, CopyHeaderInterface):
    ...     input_spec = ZerofileInputSpec
    ...     output_spec = ZerofileOutputSpec
    ...     _copy_header_map = {'out_file': 'in_file'}
    ...
    ...     def _run_interface(self, runtime):
    ...         img = nb.load(self.inputs.in_file)
    ...         # Just set the data. Let the CopyHeaderInterface mixin fix the affine and header.
    ...         nb.Nifti1Image(np.zeros(img.shape, dtype=np.uint8), None).to_filename('out.nii')
    ...         self._results = {'out_file':  os.path.abspath('out.nii')}
    ...         return runtime

    Consider a file of all ones and a non-trivial affine:

    >>> in_file = 'test.nii'
    >>> nb.Nifti1Image(np.ones((5,5,5), dtype=np.int16),
    ...                affine=np.diag((4, 3, 2, 1))).to_filename(in_file)

    The default behavior would produce a file with similar data:

    >>> res = ZerofileInterface(in_file=in_file).run()
    >>> out_img = nb.load(res.outputs.out_file)
    >>> out_img.shape
    (5, 5, 5)
    >>> np.all(out_img.get_fdata() == 0)
    True

    An updated data type:

    >>> out_img.get_data_dtype()
    dtype('uint8')

    But a different affine:

    >>> np.array_equal(out_img.affine, np.diag((4, 3, 2, 1)))
    False

    With ``copy_header=True``, then the affine is also equal:

    >>> res = ZerofileInterface(in_file=in_file, copy_header=True).run()
    >>> out_img = nb.load(res.outputs.out_file)
    >>> np.array_equal(out_img.affine, np.diag((4, 3, 2, 1)))
    True

    The data properties remain as expected:

    >>> out_img.shape
    (5, 5, 5)
    >>> out_img.get_data_dtype()
    dtype('uint8')
    >>> np.all(out_img.get_fdata() == 0)
    True

    By default, the data type of the output file is permitted to vary from the
    inputs. That is, the data type is preserved.
    If the data type of the original file is preferred, the ``_copy_header_map``
    can indicate the output data type should **not** be preserved by providing a
    tuple of the input and ``False``.

    >>> ZerofileInterface._copy_header_map['out_file'] = ('in_file', False)

    >>> res = ZerofileInterface(in_file=in_file, copy_header=True).run()
    >>> out_img = nb.load(res.outputs.out_file)
    >>> out_img.get_data_dtype()
    dtype('<i2')

    Again, the affine is updated.

    >>> np.array_equal(out_img.affine, np.diag((4, 3, 2, 1)))
    True
    >>> out_img.shape
    (5, 5, 5)
    >>> np.all(out_img.get_fdata() == 0)
    True

    Providing a tuple where the second value is ``True`` is also permissible to
    achieve the default behavior.

    """

    _copy_header_map = None

    def _post_run_hook(self, runtime):
        """Copy headers for outputs, if required."""
        runtime = super()._post_run_hook(runtime)

        if self._copy_header_map is None or not self.inputs.copy_header:
            return runtime

        inputs = self.inputs.get_traitsfree()
        outputs = self.aggregate_outputs(runtime=runtime).get_traitsfree()
        defined_outputs = set(outputs.keys()).intersection(self._copy_header_map.keys())
        for out in defined_outputs:
            inp = self._copy_header_map[out]
            keep_dtype = True
            if isinstance(inp, tuple):
                inp, keep_dtype = inp
            _copy_header(inputs[inp], outputs[out], keep_dtype=keep_dtype)

        return runtime



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brainsuite import Bse


def test_Bse_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        diffusionConstant=dict(
            argstr="-d %f",
            usedefault=True,
        ),
        diffusionIterations=dict(
            argstr="-n %d",
            usedefault=True,
        ),
        dilateFinalMask=dict(
            argstr="-p",
            usedefault=True,
        ),
        edgeDetectionConstant=dict(
            argstr="-s %f",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRIFile=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        noRotate=dict(
            argstr="--norotate",
        ),
        outputCortexFile=dict(
            argstr="--cortex %s",
            extensions=None,
            hash_files=False,
        ),
        outputDetailedBrainMask=dict(
            argstr="--hires %s",
            extensions=None,
            hash_files=False,
        ),
        outputDiffusionFilter=dict(
            argstr="--adf %s",
            extensions=None,
            hash_files=False,
        ),
        outputEdgeMap=dict(
            argstr="--edge %s",
            extensions=None,
            hash_files=False,
        ),
        outputMRIVolume=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        outputMaskFile=dict(
            argstr="--mask %s",
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        radius=dict(
            argstr="-r %f",
            usedefault=True,
        ),
        timer=dict(
            argstr="--timer",
        ),
        trim=dict(
            argstr="--trim",
            usedefault=True,
        ),
        verbosityLevel=dict(
            argstr="-v %f",
            usedefault=True,
        ),
    )
    inputs = Bse.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Bse_outputs():
    output_map = dict(
        outputCortexFile=dict(
            extensions=None,
        ),
        outputDetailedBrainMask=dict(
            extensions=None,
        ),
        outputDiffusionFilter=dict(
            extensions=None,
        ),
        outputEdgeMap=dict(
            extensions=None,
        ),
        outputMRIVolume=dict(
            extensions=None,
        ),
        outputMaskFile=dict(
            extensions=None,
        ),
    )
    outputs = Bse.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..developer import JistLaminarVolumetricLayering


def test_JistLaminarVolumetricLayering_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inInner=dict(
            argstr="--inInner %s",
            extensions=None,
        ),
        inLayering=dict(
            argstr="--inLayering %s",
        ),
        inLayering2=dict(
            argstr="--inLayering2 %s",
        ),
        inMax=dict(
            argstr="--inMax %d",
        ),
        inMin=dict(
            argstr="--inMin %f",
        ),
        inNumber=dict(
            argstr="--inNumber %d",
        ),
        inOuter=dict(
            argstr="--inOuter %s",
            extensions=None,
        ),
        inTopology=dict(
            argstr="--inTopology %s",
        ),
        incurvature=dict(
            argstr="--incurvature %d",
        ),
        inpresmooth=dict(
            argstr="--inpresmooth %s",
        ),
        inratio=dict(
            argstr="--inratio %f",
        ),
        null=dict(
            argstr="--null %s",
        ),
        outContinuous=dict(
            argstr="--outContinuous %s",
            hash_files=False,
        ),
        outDiscrete=dict(
            argstr="--outDiscrete %s",
            hash_files=False,
        ),
        outLayer=dict(
            argstr="--outLayer %s",
            hash_files=False,
        ),
        xDefaultMem=dict(
            argstr="-xDefaultMem %d",
        ),
        xMaxProcess=dict(
            argstr="-xMaxProcess %d",
            usedefault=True,
        ),
        xPrefExt=dict(
            argstr="--xPrefExt %s",
        ),
    )
    inputs = JistLaminarVolumetricLayering.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_JistLaminarVolumetricLayering_outputs():
    output_map = dict(
        outContinuous=dict(
            extensions=None,
        ),
        outDiscrete=dict(
            extensions=None,
        ),
        outLayer=dict(
            extensions=None,
        ),
    )
    outputs = JistLaminarVolumetricLayering.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import SmoothTessellation


def test_SmoothTessellation_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        curvature_averaging_iterations=dict(
            argstr="-a %d",
        ),
        disable_estimates=dict(
            argstr="-nw",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gaussian_curvature_norm_steps=dict(
            argstr="%d",
        ),
        gaussian_curvature_smoothing_steps=dict(
            argstr=" %d",
        ),
        in_file=dict(
            argstr="%s",
            copyfile=True,
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        normalize_area=dict(
            argstr="-area",
        ),
        out_area_file=dict(
            argstr="-b %s",
            extensions=None,
        ),
        out_curvature_file=dict(
            argstr="-c %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        seed=dict(
            argstr="-seed %d",
        ),
        smoothing_iterations=dict(
            argstr="-n %d",
        ),
        snapshot_writing_iterations=dict(
            argstr="-w %d",
        ),
        subjects_dir=dict(),
        use_gaussian_curvature_smoothing=dict(
            argstr="-g",
        ),
        use_momentum=dict(
            argstr="-m",
        ),
    )
    inputs = SmoothTessellation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SmoothTessellation_outputs():
    output_map = dict(
        surface=dict(
            extensions=None,
        ),
    )
    outputs = SmoothTessellation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..base import Rename


def test_Rename_inputs():
    input_map = dict(
        format_string=dict(
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        keep_ext=dict(),
        parse_string=dict(),
        use_fullpath=dict(
            usedefault=True,
        ),
    )
    inputs = Rename.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Rename_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Rename.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



class Cell:
    def __init__(self, arg):
        self.arg = arg

    def to_string(self):
        if isinstance(self.arg, list):
            v = "\n".join([f"'{el}'" for el in self.arg])
        else:
            v = self.arg
        return v


class NestedCell(Cell):
    def __str__(self):
        return "{{%s}}" % self.to_string()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..stats import ActivationCount


def test_ActivationCount_inputs():
    input_map = dict(
        in_files=dict(
            mandatory=True,
        ),
        threshold=dict(
            mandatory=True,
        ),
    )
    inputs = ActivationCount.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ActivationCount_outputs():
    output_map = dict(
        acm_neg=dict(
            extensions=None,
        ),
        acm_pos=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = ActivationCount.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..tensors import GenerateDirections


def test_GenerateDirections_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        display_debug=dict(
            argstr="-debug",
        ),
        display_info=dict(
            argstr="-info",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        niter=dict(
            argstr="-niter %s",
        ),
        num_dirs=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            hash_files=False,
            name_source=["num_dirs"],
            name_template="directions_%d.txt",
            position=-1,
        ),
        power=dict(
            argstr="-power %s",
        ),
        quiet_display=dict(
            argstr="-quiet",
        ),
    )
    inputs = GenerateDirections.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_GenerateDirections_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = GenerateDirections.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import Average


def test_Average_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        avgdim=dict(
            argstr="-avgdim %s",
        ),
        binarize=dict(
            argstr="-binarize",
        ),
        binrange=dict(
            argstr="-binrange %s %s",
        ),
        binvalue=dict(
            argstr="-binvalue %s",
        ),
        check_dimensions=dict(
            argstr="-check_dimensions",
            xor=("check_dimensions", "no_check_dimensions"),
        ),
        clobber=dict(
            argstr="-clobber",
            usedefault=True,
        ),
        copy_header=dict(
            argstr="-copy_header",
            xor=("copy_header", "no_copy_header"),
        ),
        debug=dict(
            argstr="-debug",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filelist=dict(
            argstr="-filelist %s",
            extensions=None,
            mandatory=True,
            xor=("input_files", "filelist"),
        ),
        format_byte=dict(
            argstr="-byte",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_double=dict(
            argstr="-double",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_filetype=dict(
            argstr="-filetype",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_float=dict(
            argstr="-float",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_int=dict(
            argstr="-int",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_long=dict(
            argstr="-long",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_short=dict(
            argstr="-short",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_signed=dict(
            argstr="-signed",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        format_unsigned=dict(
            argstr="-unsigned",
            xor=(
                "format_filetype",
                "format_byte",
                "format_short",
                "format_int",
                "format_long",
                "format_float",
                "format_double",
                "format_signed",
                "format_unsigned",
            ),
        ),
        input_files=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
            sep=" ",
            xor=("input_files", "filelist"),
        ),
        max_buffer_size_in_kb=dict(
            argstr="-max_buffer_size_in_kb %d",
            usedefault=True,
        ),
        no_check_dimensions=dict(
            argstr="-nocheck_dimensions",
            xor=("check_dimensions", "no_check_dimensions"),
        ),
        no_copy_header=dict(
            argstr="-nocopy_header",
            xor=("copy_header", "no_copy_header"),
        ),
        nonormalize=dict(
            argstr="-nonormalize",
            xor=("normalize", "nonormalize"),
        ),
        normalize=dict(
            argstr="-normalize",
            xor=("normalize", "nonormalize"),
        ),
        output_file=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=["input_files"],
            name_template="%s_averaged.mnc",
            position=-1,
        ),
        quiet=dict(
            argstr="-quiet",
            xor=("verbose", "quiet"),
        ),
        sdfile=dict(
            argstr="-sdfile %s",
            extensions=None,
        ),
        two=dict(
            argstr="-2",
        ),
        verbose=dict(
            argstr="-verbose",
            xor=("verbose", "quiet"),
        ),
        voxel_range=dict(
            argstr="-range %d %d",
        ),
        weights=dict(
            argstr="-weights %s",
            sep=",",
        ),
        width_weighted=dict(
            argstr="-width_weighted",
            requires=("avgdim",),
        ),
    )
    inputs = Average.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Average_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = Average.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import os

import pytest
import nipype.testing as npt
from nipype.testing import example_data
import numpy as np
from nipype.algorithms import mesh as m
from ...interfaces import vtkbase as VTKInfo


@pytest.mark.skipif(VTKInfo.no_tvtk(), reason="tvtk is not installed")
def test_ident_distances(tmpdir):
    tmpdir.chdir()

    in_surf = example_data("surf01.vtk")
    dist_ident = m.ComputeMeshWarp()
    dist_ident.inputs.surface1 = in_surf
    dist_ident.inputs.surface2 = in_surf
    dist_ident.inputs.out_file = tmpdir.join("distance.npy").strpath
    res = dist_ident.run()
    assert res.outputs.distance == 0.0

    dist_ident.inputs.weighting = "area"
    res = dist_ident.run()
    assert res.outputs.distance == 0.0


@pytest.mark.skipif(VTKInfo.no_tvtk(), reason="tvtk is not installed")
def test_trans_distances(tmpdir):
    from ...interfaces.vtkbase import tvtk

    in_surf = example_data("surf01.vtk")
    warped_surf = tmpdir.join("warped.vtk").strpath

    inc = np.array([0.7, 0.3, -0.2])

    r1 = tvtk.PolyDataReader(file_name=in_surf)
    vtk1 = VTKInfo.vtk_output(r1)
    r1.update()
    vtk1.points = np.array(vtk1.points) + inc

    writer = tvtk.PolyDataWriter(file_name=warped_surf)
    VTKInfo.configure_input_data(writer, vtk1)
    writer.write()

    dist = m.ComputeMeshWarp()
    dist.inputs.surface1 = in_surf
    dist.inputs.surface2 = warped_surf
    dist.inputs.out_file = tmpdir.join("distance.npy").strpath
    res = dist.run()
    assert np.allclose(res.outputs.distance, np.linalg.norm(inc), 4)
    dist.inputs.weighting = "area"
    res = dist.run()
    assert np.allclose(res.outputs.distance, np.linalg.norm(inc), 4)


@pytest.mark.skipif(VTKInfo.no_tvtk(), reason="tvtk is not installed")
def test_warppoints(tmpdir):
    tmpdir.chdir()

    # TODO: include regression tests for when tvtk is installed


@pytest.mark.skipif(VTKInfo.no_tvtk(), reason="tvtk is not installed")
def test_meshwarpmaths(tmpdir):
    tmpdir.chdir()

    # TODO: include regression tests for when tvtk is installed


@pytest.mark.skipif(not VTKInfo.no_tvtk(), reason="tvtk is installed")
def test_importerror():
    with pytest.raises(ImportError):
        m.ComputeMeshWarp()

    with pytest.raises(ImportError):
        m.WarpPoints()

    with pytest.raises(ImportError):
        m.MeshWarpMaths()



# -*- coding: utf-8 -*-
"""Diffusion Toolkit performs data reconstruction and fiber tracking on diffusion MR images."""
from .base import Info
from .postproc import SplineFilter, TrackMerge
from .dti import DTIRecon, DTITracker
from .odf import HARDIMat, ODFRecon, ODFTracker
